#!perl
use Test::More;
eval "use Test::Perl::Critic";
plan skip_all => "Test::Perl::Critic required for testing PBP compliance" if $@;

Test::Perl::Critic->import(-profile => 't/perlcritic.rc');
Test::Perl::Critic::all_critic_ok();
